"""
Astronomical calculations for target tracking.

This module provides time system conversions and astronomical utilities
commonly needed in tracking applications.
"""

from pytcl.astronomical.time_systems import (
    # Julian dates
    cal_to_jd,
    jd_to_cal,
    mjd_to_jd,
    jd_to_mjd,
    # Time scales
    utc_to_tai,
    tai_to_utc,
    tai_to_tt,
    tt_to_tai,
    utc_to_tt,
    tt_to_utc,
    tai_to_gps,
    gps_to_tai,
    utc_to_gps,
    gps_to_utc,
    # Unix time
    unix_to_jd,
    jd_to_unix,
    # GPS week
    gps_week_seconds,
    gps_week_to_utc,
    # Sidereal time
    gmst,
    gast,
    # Leap seconds
    get_leap_seconds,
    LeapSecondTable,
)

__all__ = [
    # Julian dates
    "cal_to_jd",
    "jd_to_cal",
    "mjd_to_jd",
    "jd_to_mjd",
    # Time scales
    "utc_to_tai",
    "tai_to_utc",
    "tai_to_tt",
    "tt_to_tai",
    "utc_to_tt",
    "tt_to_utc",
    "tai_to_gps",
    "gps_to_tai",
    "utc_to_gps",
    "gps_to_utc",
    # Unix time
    "unix_to_jd",
    "jd_to_unix",
    # GPS week
    "gps_week_seconds",
    "gps_week_to_utc",
    # Sidereal time
    "gmst",
    "gast",
    # Leap seconds
    "get_leap_seconds",
    "LeapSecondTable",
]



"""
Mathematical functions and utilities.

This module contains a wide variety of mathematical functions including:
- Basic matrix operations
- Combinatorics (permutations, combinations)
- Continuous optimization
- Geometry primitives
- Interpolation methods
- Numerical integration
- Polynomials
- Signal processing
- Special functions
- Statistics and distributions
"""

# Import submodules for easy access
from pytcl.mathematical_functions import basic_matrix
from pytcl.mathematical_functions import special_functions
from pytcl.mathematical_functions import statistics
from pytcl.mathematical_functions import numerical_integration
from pytcl.mathematical_functions import interpolation
from pytcl.mathematical_functions import combinatorics
from pytcl.mathematical_functions import geometry

# Re-export commonly used functions at the top level for convenience

# Basic matrix operations
from pytcl.mathematical_functions.basic_matrix import (
    chol_semi_def,
    tria,
    tria_sqrt,
    pinv_truncated,
    matrix_sqrt,
    null_space,
    range_space,
    block_diag,
    kron,
    vec,
    unvec,
)

# Special functions
from pytcl.mathematical_functions.special_functions import (
    gamma,
    gammaln,
    beta,
    betaln,
    erf,
    erfc,
    erfinv,
    besselj,
    bessely,
    besseli,
    besselk,
)

# Statistics
from pytcl.mathematical_functions.statistics import (
    Gaussian,
    MultivariateGaussian,
    Uniform,
    ChiSquared,
    nees,
    nis,
    weighted_mean,
    weighted_cov,
    mad,
)

# Numerical integration
from pytcl.mathematical_functions.numerical_integration import (
    gauss_legendre,
    gauss_hermite,
    quad,
    spherical_cubature,
    unscented_transform_points,
)

# Interpolation
from pytcl.mathematical_functions.interpolation import (
    interp1d,
    linear_interp,
    cubic_spline,
    interp2d,
    rbf_interpolate,
)

# Combinatorics
from pytcl.mathematical_functions.combinatorics import (
    factorial,
    n_choose_k,
    permutations,
    combinations,
    permutation_rank,
    permutation_unrank,
)

# Geometry
from pytcl.mathematical_functions.geometry import (
    point_in_polygon,
    convex_hull,
    polygon_area,
    line_intersection,
    bounding_box,
)

__all__ = [
    # Submodules
    "basic_matrix",
    "special_functions",
    "statistics",
    "numerical_integration",
    "interpolation",
    "combinatorics",
    "geometry",
    # Basic matrix
    "chol_semi_def",
    "tria",
    "tria_sqrt",
    "pinv_truncated",
    "matrix_sqrt",
    "null_space",
    "range_space",
    "block_diag",
    "kron",
    "vec",
    "unvec",
    # Special functions
    "gamma",
    "gammaln",
    "beta",
    "betaln",
    "erf",
    "erfc",
    "erfinv",
    "besselj",
    "bessely",
    "besseli",
    "besselk",
    # Statistics
    "Gaussian",
    "MultivariateGaussian",
    "Uniform",
    "ChiSquared",
    "nees",
    "nis",
    "weighted_mean",
    "weighted_cov",
    "mad",
    # Numerical integration
    "gauss_legendre",
    "gauss_hermite",
    "quad",
    "spherical_cubature",
    "unscented_transform_points",
    # Interpolation
    "interp1d",
    "linear_interp",
    "cubic_spline",
    "interp2d",
    "rbf_interpolate",
    # Combinatorics
    "factorial",
    "n_choose_k",
    "permutations",
    "combinations",
    "permutation_rank",
    "permutation_unrank",
    # Geometry
    "point_in_polygon",
    "convex_hull",
    "polygon_area",
    "line_intersection",
    "bounding_box",
]



"""
End-to-end tracker implementations.

This module provides complete tracker implementations that combine
filtering, data association, and track management.
"""

from pytcl.trackers.single_target import (
    SingleTargetTracker,
    TrackState,
)

from pytcl.trackers.multi_target import (
    MultiTargetTracker,
    Track,
    TrackStatus,
)

__all__ = [
    "SingleTargetTracker",
    "TrackState",
    "MultiTargetTracker",
    "Track",
    "TrackStatus",
]



"""
Tracker Component Library - Python Port

A comprehensive library for target tracking algorithms, including coordinate
systems, dynamic models, estimation algorithms, and mathematical functions.

This is a Python port of the U.S. Naval Research Laboratory's Tracker Component
Library originally written in MATLAB.

Examples
--------
>>> import pytcl as pytcl
>>> from pytcl.coordinate_systems import cart2sphere
>>> from pytcl.dynamic_estimation.kalman import KalmanFilter

References
----------
.. [1] D. F. Crouse, "The Tracker Component Library: Free Routines for Rapid
       Prototyping," IEEE Aerospace and Electronic Systems Magazine, vol. 32,
       no. 5, pp. 18-27, May 2017.
"""

__version__ = "0.2.1"
__author__ = "Python Port Contributors"
__original_author__ = "David F. Crouse, Naval Research Laboratory"

# Core utilities
from pytcl import core

# Assignment algorithms (Phase 5)
from pytcl import assignment_algorithms

# Specialized domains (Phase 6)
from pytcl import astronomical
from pytcl import navigation
from pytcl import atmosphere

# End-to-end trackers (Phase 7)
from pytcl import trackers

# Performance evaluation
from pytcl import performance_evaluation

# Plotting utilities
from pytcl import plotting

# Version tuple for programmatic access
VERSION = tuple(int(x) for x in __version__.split("."))

__all__ = [
    "__version__",
    "__author__",
    "__original_author__",
    "VERSION",
    "core",
    "assignment_algorithms",
    "astronomical",
    "navigation",
    "atmosphere",
    "trackers",
    "performance_evaluation",
    "plotting",
]



"""
Kalman filter family implementations.

This module provides:
- Linear Kalman filter (predict, update, smoothing)
- Extended Kalman filter (EKF)
- Unscented Kalman filter (UKF)
- Cubature Kalman filter (CKF)
- Information filter
"""

from pytcl.dynamic_estimation.kalman.linear import (
    KalmanState,
    KalmanPrediction,
    KalmanUpdate,
    kf_predict,
    kf_update,
    kf_predict_update,
    kf_smooth,
    information_filter_predict,
    information_filter_update,
)

from pytcl.dynamic_estimation.kalman.extended import (
    ekf_predict,
    ekf_update,
    numerical_jacobian,
    ekf_predict_auto,
    ekf_update_auto,
    iterated_ekf_update,
)

from pytcl.dynamic_estimation.kalman.unscented import (
    SigmaPoints,
    sigma_points_merwe,
    sigma_points_julier,
    unscented_transform,
    ukf_predict,
    ukf_update,
    ckf_spherical_cubature_points,
    ckf_predict,
    ckf_update,
)

__all__ = [
    # Linear KF
    "KalmanState",
    "KalmanPrediction",
    "KalmanUpdate",
    "kf_predict",
    "kf_update",
    "kf_predict_update",
    "kf_smooth",
    "information_filter_predict",
    "information_filter_update",
    # EKF
    "ekf_predict",
    "ekf_update",
    "numerical_jacobian",
    "ekf_predict_auto",
    "ekf_update_auto",
    "iterated_ekf_update",
    # UKF
    "SigmaPoints",
    "sigma_points_merwe",
    "sigma_points_julier",
    "unscented_transform",
    "ukf_predict",
    "ukf_update",
    # CKF
    "ckf_spherical_cubature_points",
    "ckf_predict",
    "ckf_update",
]
